NODE_DEPOSIT = 100000000000000000000

GAS = {
    'create_node': 4500000,
    'get_bounty': 2000000,
    'send_verdict': 200000,

    'token_transfer': 30000,
    'add_authorized': 100000
}

OP_TYPES = {
    'create_node': 0x1,
    'create_schain': 0x10
}
